#!/usr/bin/env python3
# gcalendar is a tool to read Google Calendar events from your terminal.

# Copyright (C) 2020  Gobinath

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

GOOGLE_CALENDAR_SCOPE = "https://www.googleapis.com/auth/calendar.readonly"

AUTH_URI = "https://accounts.google.com/o/oauth2/auth"

TOKEN_URI = "https://accounts.google.com/o/oauth2/token"

DEFAULT_CLIENT_ID = "1097830982036-5opi3qc4lje5ak0so1mjg0icktiiu289.apps.googleusercontent.com"

DEFAULT_CLIENT_SECRET = "S5zEdU60nQfdaS3AXXTLNivB"
